package com.sxtvs.open.api.bid.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sxtvs.open.api.bid.dto.FileImportDTO;
import com.sxtvs.open.api.bid.dto.PageKeywordDTO;
import com.sxtvs.open.api.bid.entity.BidProject;
import com.sxtvs.open.api.bid.mapper.BidProjectMapper;
import com.sxtvs.open.api.bid.service.IBidProjectService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sxtvs.open.api.h5template.service.impl.OSSService;
import com.sxtvs.open.core.advice.BizException;
import com.sxtvs.open.core.auth.AESUtil;
import com.sxtvs.open.core.auth.HttpContextUtil;
import jakarta.annotation.Resource;
import org.apache.http.util.TextUtils;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * <p>
 * 标书项目 服务实现类
 * </p>
 *
 * @author syj
 * @since 2024-04-23
 */
@Service
public class BidProjectServiceImpl extends ServiceImpl<BidProjectMapper, BidProject> implements IBidProjectService {


    @Resource
    private BidFilesServiceImpl bidFilesService;

    @Resource
    private StringRedisTemplate stringRedisTemplate;

    @Resource
    private OSSService ossService;


    public void create(BidProject bidProject)
    {
        Optional<String> token = HttpContextUtil.getToken();
        Long uid = Long.valueOf(AESUtil.decryptStr(token.get()));
        bidProject.setUid(uid);
        bidProject.setCreateTime(LocalDateTime.now());
        this.save(bidProject);
    }

    public void update(BidProject bidProject)
    {
        BidProject bp = getById(bidProject.getId());
        bp.setProjectName(bidProject.getProjectName());
        this.updateById(bidProject);
    }

    @Transactional
    public void delete(Long id)
    {
        bidFilesService.deleteByProjectId(id);
        this.removeById(id);
        ossService.bidDel(id);
    }

    public Page<BidProject> bidProjectPage(PageKeywordDTO pageKeywordDTO){
        Optional<String> token = HttpContextUtil.getToken();
        Long uid = Long.valueOf(AESUtil.decryptStr(token.get()));
        Page<BidProject> objectPage = new Page<>(pageKeywordDTO.getPage(), pageKeywordDTO.getPageSize());
        LambdaQueryWrapper<BidProject> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(BidProject::getUid, uid);
        if(!TextUtils.isEmpty(pageKeywordDTO.getKeyword())){
            wrapper.like(BidProject::getProjectName, pageKeywordDTO.getKeyword());
        }
        Page<BidProject> page = this.page(objectPage, wrapper);
        page.setRecords(page.getRecords().stream()
                .peek(bidProject -> bidProject.setBidFiles(bidFilesService.byProject(bidProject.getId())))
                .collect(Collectors.toList()));
        return page;
    }

    @Transactional
    public void fileImport(FileImportDTO fileImportDTO){

        BidProject bidProject = getById(fileImportDTO.getProjectId());

        if (bidProject.getStatus() > 0){
            throw new BizException("请勿重复上传");
        }

        bidProject.setStatus(1);
        updateById(bidProject);

        bidFilesService.create(fileImportDTO);
        stringRedisTemplate.convertAndSend("bid_compare", String.valueOf(fileImportDTO.getProjectId()));
    }


}
